@@ -11,8 +11,11 @@ from django.core import mail
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.core.exceptions import ValidationError
+from django.db.models import Q
 from django.test import TestCase
 from django.test.client import Client
+from django.test.utils import override_settings
+from pybb import permissions, views as pybb_views
 from pybb.templatetags.pybb_tags import pybb_is_topic_unread, pybb_topic_unread, pybb_forum_unread, \
     pybb_get_latest_topics, pybb_get_latest_posts
 
@@ -45,7 +48,7 @@ class SharedTestModule(object):
         self.forum = Forum.objects.create(name='xfoo', description='bar', category=self.category)
         self.topic = Topic.objects.create(name='etopic', forum=self.forum, user=self.user)
         if post:
-            self.post = Post.objects.create(topic=self.topic, user=self.user, body='bbcode [b]test[b]')
+            self.post = Post.objects.create(topic=self.topic, user=self.user, body='bbcode [b]test[/b]')
 
     def get_form_values(self, response, form="post-form"):
         return dict(html.fromstring(response.content).xpath('//form[@class="%s"]' % form)[0].form_values())
@@ -129,6 +132,26 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertEqual(response.context['paginator'].num_pages,
                          int((defaults.PYBB_FORUM_PAGE_SIZE + 3) / defaults.PYBB_FORUM_PAGE_SIZE) + 1)
 
+    def test_default_bbcode_processor(self):
+        bbcode_to_html_map = [
+            ['[b]bold[/b]', '<strong>bold</strong>'],
+            ['[i]italic[/i]', '<em>italic</em>'],
+            ['[u]underline[/u]', '<u>underline</u>'],
+            ['[s]striked[/s]', '<strike>striked</strike>'],
+            ['[img]http://domain.com/image.png[/img]', '<img src="http://domain.com/image.png"></img>',
+                                                       '<img src="http://domain.com/image.png">'],
+            ['[url=google.com]search in google[/url]', '<a href="http://google.com">search in google</a>'],
+            ['[list][*]1[*]2[/list]', '<ul><li>1</li><li>2</li></ul>'],
+            ['[list=1][*]1[*]2[/list]', '<ol><li>1</li><li>2</li></ol>',
+                                        '<ol style="list-style-type:decimal;"><li>1</li><li>2</li></ol>'],
+            ['[quote]quote[/quote]', '<blockquote>quote</blockquote>'],
+            ['[code]code[/code]', '<div class="code"><pre>code</pre></div>',
+                                  '<code>code</code>'],
+        ]
+
+        for item in bbcode_to_html_map:
+            self.assertIn(defaults.PYBB_MARKUP_ENGINES['bbcode'](item[0]), item[1:])
+
     def test_bbcode_and_topic_title(self):
         response = self.client.get(self.topic.get_absolute_url())
         tree = html.fromstring(response.content)
@@ -149,7 +172,7 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertTrue(Topic.objects.filter(name='new topic name').exists())
 
     def test_post_deletion(self):
-        post = Post(topic=self.topic, user=self.user, body='bbcode [b]test[b]')
+        post = Post(topic=self.topic, user=self.user, body='bbcode [b]test[/b]')
         post.save()
         post.delete()
         Topic.objects.get(id=self.topic.id)
@@ -728,7 +751,7 @@ class FeaturesTest(TestCase, SharedTestModule):
     def test_user_blocking(self):
         user = User.objects.create_user('test', 'test@localhost', 'test')
         topic = Topic.objects.create(name='topic', forum=self.forum, user=self.user)
-        self.post = Post.objects.create(topic=topic, user=user, body='bbcode [b]test[b]')
+        self.post = Post.objects.create(topic=topic, user=user, body='bbcode [b]test[/b]')
         self.user.is_superuser = True
         self.user.save()
         self.login_client()
@@ -767,7 +790,7 @@ class FeaturesTest(TestCase, SharedTestModule):
 
     def test_ajax_preview(self):
         self.login_client()
-        response = self.client.post(reverse('pybb:post_ajax_preview'), data={'data': '[b]test bbcode ajax preview[b]'})
+        response = self.client.post(reverse('pybb:post_ajax_preview'), data={'data': '[b]test bbcode ajax preview[/b]'})
         self.assertContains(response, '<strong>test bbcode ajax preview</strong>')
 
     def test_headline(self):
@@ -898,13 +921,13 @@ class FeaturesTest(TestCase, SharedTestModule):
         topic = Topic(name='etopic', forum=self.forum, user=self.user)
         topic.save()
         time.sleep(1)
-        post = Post(topic=topic, user=self.user, body='bbcode [b]test[b]')
+        post = Post(topic=topic, user=self.user, body='bbcode [b]test[/b]')
         post.save()
         client = Client()
         response = client.get(self.forum.get_absolute_url())
         self.assertEqual(response.context['topic_list'][0], topic)
         time.sleep(1)
-        post = Post(topic=self.topic, user=self.user, body='bbcode [b]test[b]')
+        post = Post(topic=self.topic, user=self.user, body='bbcode [b]test[/b]')
         post.save()
         client = Client()
         response = client.get(self.forum.get_absolute_url())
@@ -1420,10 +1443,6 @@ class FiltersTest(TestCase, SharedTestModule):
         self.assertEqual(Post.objects.all()[0].body, 'test\nmultiple empty lines')
 
 
-from pybb import permissions
-from django.db.models import Q
-
-
 class CustomPermissionHandler(permissions.DefaultPermissionHandler):
     """ 
     a custom permission handler which changes the meaning of "hidden" forum:
@@ -1465,12 +1484,26 @@ class CustomPermissionHandler(permissions.DefaultPermissionHandler):
         return False
 
 
+def _attach_perms_class(class_name):
+    """
+    override the permission handler. this cannot be done with @override_settings as
+    permissions.perms is already imported at import point, instead we got to monkeypatch
+    the modules (not really nice, but only an issue in tests)
+    """
+    pybb_views.perms = permissions.perms = permissions._resolve_class(class_name)
+
+
+def _detach_perms_class():
+    """
+    reset permission handler (otherwise other tests may fail)
+    """
+    pybb_views.perms = permissions.perms = permissions._resolve_class('pybb.permissions.DefaultPermissionHandler')
+
+
 class CustomPermissionHandlerTest(TestCase, SharedTestModule):
     """ test custom permission handler """
 
     def setUp(self):
-        from pybb import views
-
         self.create_user()
         # create public and hidden categories, forums, posts
         c_pub = Category(name='public')
@@ -1487,15 +1520,11 @@ class CustomPermissionHandlerTest(TestCase, SharedTestModule):
         for t in Topic.objects.all()[0:2]:
             t.closed = True
             t.save()
-        # override the permission handler. this cannot be done with @override_settings as
-        # permissions.perms is already imported at this point, instead we got to monkeypatch
-        # the modules (not really nice, but only an issue in tests)
-        views.perms = permissions.perms = permissions._resolve_class('pybb.tests.CustomPermissionHandler')
+
+        _attach_perms_class('pybb.tests.CustomPermissionHandler')
 
     def tearDown(self):
-        from pybb import views
-        # reset permission handler (otherwise other tests may fail)
-        views.perms = permissions.perms = permissions._resolve_class('pybb.permissions.DefaultPermissionHandler')
+        _detach_perms_class()
 
     def test_category_permission(self):
         for c in Category.objects.all():
@@ -1550,6 +1579,17 @@ class CustomPermissionHandlerTest(TestCase, SharedTestModule):
         self.assertFalse(PollAnswer.objects.filter(topic=new_topic).exists()) # no answers here
 
 
+class RestrictEditingHandler(permissions.DefaultPermissionHandler):
+        def may_create_topic(self, user, forum):
+            return False
+
+        def may_create_post(self, user, topic):
+            return False
+
+        def may_edit_post(self, user, post):
+            return False
+
+
 class LogonRedirectTest(TestCase, SharedTestModule):
     """ test whether anonymous user gets redirected, whereas unauthorized user gets PermissionDenied """
 
@@ -1616,4 +1656,40 @@ class LogonRedirectTest(TestCase, SharedTestModule):
         r = self.get_with_user(self.post.get_absolute_url(), 'staff', 'staff')
         self.assertEquals(r.status_code, 301)
 
-    
+    @override_settings(PYBB_ENABLE_ANONYMOUS_POST=False)
+    def test_redirect_topic_add(self):
+        _attach_perms_class('pybb.tests.RestrictEditingHandler')
+
+        # access without user should be redirected
+        add_topic_url = reverse('pybb:add_topic', kwargs={'forum_id': self.forum.id})
+        r = self.get_with_user(add_topic_url)
+        self.assertRedirects(r, settings.LOGIN_URL + '?next=%s' % add_topic_url)
+
+        # access with (unauthorized) user should get 403 (forbidden)
+        r = self.get_with_user(add_topic_url, 'staff', 'staff')
+        self.assertEquals(r.status_code, 403)
+
+        _detach_perms_class()
+
+        # allowed user is allowed
+        r = self.get_with_user(add_topic_url, 'staff', 'staff')
+        self.assertEquals(r.status_code, 200)
+
+    def test_redirect_post_edit(self):
+        _attach_perms_class('pybb.tests.RestrictEditingHandler')
+
+        # access without user should be redirected
+        edit_post_url = reverse('pybb:edit_post', kwargs={'pk': self.post.id})
+        r = self.get_with_user(edit_post_url)
+        self.assertRedirects(r, settings.LOGIN_URL + '?next=%s' % edit_post_url)
+
+        # access with (unauthorized) user should get 403 (forbidden)
+        r = self.get_with_user(edit_post_url, 'staff', 'staff')
+        self.assertEquals(r.status_code, 403)
+
+        _detach_perms_class()
+
+        # allowed user is allowed
+        r = self.get_with_user(edit_post_url, 'staff', 'staff')
+        self.assertEquals(r.status_code, 200)
+
